@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2024	PgPool Global Development Group
+ * Copyright (c) 2003-2020	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -45,19 +45,28 @@
 #include <sys/time.h>
 
 #include "pool.h"
-#include "utils/palloc.h"
-#include "utils/memutils.h"
-#include "context/pool_process_context.h"
-#include "context/pool_session_context.h"
 #include "pool_config.h"
 #include "pool_config_variables.h"
+#include "utils/palloc.h"
+#include "utils/memutils.h"
+#include "utils/pool_ssl.h"
+#include "utils/pool_ipc.h"
+#include "utils/pool_relcache.h"
 #include "utils/pool_ip.h"
 #include "utils/pool_stream.h"
 #include "utils/elog.h"
+#include "utils/ps_status.h"
+#include "utils/timestamp.h"
+
+#include "context/pool_process_context.h"
+#include "context/pool_session_context.h"
+#include "protocol/pool_connection_pool.h"
+#include "protocol/pool_process_query.h"
+#include "protocol/pool_pg_utils.h"
+#include "auth/pool_auth.h"
 #include "auth/md5.h"
 #include "auth/pool_passwd.h"
 #include "auth/pool_hba.h"
-#include "utils/pool_relcache.h"
 
 static StartupPacket *read_startup_packet(POOL_CONNECTION * cp);
 static POOL_CONNECTION_POOL * connect_backend(StartupPacket *sp, POOL_CONNECTION * frontend);
@@ -67,7 +76,6 @@ static RETSIGTYPE wakeup_handler(int sig);
 static RETSIGTYPE reload_config_handler(int sig);
 static RETSIGTYPE authentication_timeout(int sig);
 static void send_params(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend);
-static void send_frontend_exits(void);
 static int	connection_count_up(void);
 static void connection_count_down(void);
 static bool connect_using_existing_connection(POOL_CONNECTION * frontend,
@@ -83,10 +91,10 @@ static void validate_backend_connectivity(int front_end_fd);
 static POOL_CONNECTION * get_connection(int front_end_fd, SockAddr *saddr);
 static POOL_CONNECTION_POOL * get_backend_connection(POOL_CONNECTION * frontend);
 static StartupPacket *StartupPacketCopy(StartupPacket *sp);
+static void log_disconnections(char *database, char *username);
 static void print_process_status(char *remote_host, char *remote_port);
 static bool backend_cleanup(POOL_CONNECTION * volatile *frontend, POOL_CONNECTION_POOL * volatile backend, bool frontend_invalid);
-static void free_persisten_db_connection_memory(POOL_CONNECTION_POOL_SLOT * cp);
-static int	choose_db_node_id(char *str);
+
 static void child_will_go_down(int code, Datum arg);
 static int opt_sort(const void *a, const void *b);
 
@@ -101,6 +109,11 @@ static volatile sig_atomic_t alarm_enabled = false;
  */
 volatile sig_atomic_t ignore_sigusr1 = 0;
 
+/*
+ * si modules use SIGUSR2
+ */
+volatile sig_atomic_t sigusr2_received = 0;
+
 static int	idle;				/* non 0 means this child is in idle state */
 static int	accepted = 0;
 
@@ -112,14 +125,14 @@ static int	child_unix_fd = 0;
 extern int	myargc;
 extern char **myargv;
 
-char		remote_ps_data[NI_MAXHOST + NI_MAXSERV + 2]; /* used for set_ps_display */
-
 volatile sig_atomic_t got_sighup = 0;
 
 char		remote_host[NI_MAXHOST];	/* client host */
 char		remote_port[NI_MAXSERV];	/* client port */
 POOL_CONNECTION *volatile child_frontend = NULL;
 
+struct timeval startTime;
+
 #ifdef DEBUG
 bool		stop_now = false;
 #endif
@@ -137,13 +150,7 @@ do_child(int *fds)
 	struct timeval timeout;
 	static int	connected = 0;	/* non 0 if has been accepted connections from
 								 * frontend */
-
-	/* counter for child_max_connections.  "volatile" declaration is necessary
-	 * so that this is counted up even if long jump is issued due to
-	 * ereport(ERROR).
-	 */
-	volatile	int			connections_count = 0;
-
+	int			connections_count = 0;	/* used if child_max_connections > 0 */
 	char		psbuf[NI_MAXHOST + 128];
 
 	ereport(DEBUG2,
@@ -169,7 +176,7 @@ do_child(int *fds)
 #ifdef NONE_BLOCK
 	/* set listen fds to none-blocking */
 	for (walk = fds; *walk != -1; walk++)
-		pool_set_nonblock(*walk);
+		socket_set_nonblock(*walk);
 #endif
 	for (walk = fds; *walk != -1; walk++)
 	{
@@ -251,6 +258,8 @@ do_child(int *fds)
 		{
 			accepted = 0;
 			connection_count_down();
+			if (pool_config->log_disconnections)
+				log_disconnections(child_frontend->database, child_frontend->username);
 		}
 
 		backend_cleanup(&child_frontend, backend, frontend_invalid);
@@ -331,44 +340,41 @@ do_child(int *fds)
 		if (front_end_fd == RETRY)
 			continue;
 
+		/*
+		 * Check if max connections from clients execeeded.
+		 */
 		con_count = connection_count_up();
-
-		if (pool_config->reserved_connections > 0)
+		if (con_count > (pool_config->num_init_children - pool_config->reserved_connections))
 		{
-			/*
-			 * Check if max connections from clients exceeded.
-			 */
-			if (con_count > (pool_config->num_init_children - pool_config->reserved_connections))
+			POOL_CONNECTION * cp;
+			cp = pool_open(front_end_fd, false);
+			if (cp == NULL)
 			{
-				POOL_CONNECTION * cp;
-				cp = pool_open(front_end_fd, false);
-				if (cp == NULL)
-				{
-					connection_count_down();
-					continue;
-				}
 				connection_count_down();
-				pool_send_fatal_message(cp, 3, "53300",
-										"Sorry, too many clients already",
-										"",
-										"",
-										__FILE__, __LINE__);
-				ereport(ERROR,
-						(errcode(ERRCODE_TOO_MANY_CONNECTIONS),
-						 errmsg("Sorry, too many clients already")));
-				pool_close(cp);
 				continue;
 			}
+			connection_count_down();
+			pool_send_fatal_message(cp, 3, "53300",
+									"Sorry, too many clients already",
+									"",
+									"",
+									__FILE__, __LINE__);
+			ereport(ERROR,
+					(errcode(ERRCODE_TOO_MANY_CONNECTIONS),
+					 errmsg("Sorry, too many clients already")));
+			pool_close(cp);
+			continue;
 		}
 
 		accepted = 1;
+		gettimeofday(&startTime, NULL);
 
 		check_config_reload();
 		validate_backend_connectivity(front_end_fd);
 		child_frontend = get_connection(front_end_fd, &saddr);
 
 		/* set frontend fd to blocking */
-		pool_unset_nonblock(child_frontend->fd);
+		socket_unset_nonblock(child_frontend->fd);
 
 		/* reset busy flag */
 		idle = 0;
@@ -383,6 +389,8 @@ do_child(int *fds)
 		backend = get_backend_connection(child_frontend);
 		if (!backend)
 		{
+			if (pool_config->log_disconnections)
+				log_disconnections(child_frontend->database, child_frontend->username);
 			pool_close(child_frontend);
 			child_frontend = NULL;
 			continue;
@@ -392,7 +400,7 @@ do_child(int *fds)
 		/*
 		 * show ps status
 		 */
-		sp = MASTER_CONNECTION(backend)->sp;
+		sp = MAIN_CONNECTION(backend)->sp;
 		snprintf(psbuf, sizeof(psbuf), "%s %s %s idle",
 				 sp->user, sp->database, remote_ps_data);
 		set_ps_display(psbuf, false);
@@ -444,6 +452,8 @@ do_child(int *fds)
 
 		accepted = 0;
 		connection_count_down();
+		if (pool_config->log_disconnections)
+			log_disconnections(sp->database, sp->user);
 
 		timeout.tv_sec = pool_config->child_life_time;
 		timeout.tv_usec = 0;
@@ -482,7 +492,7 @@ backend_cleanup(POOL_CONNECTION * volatile *frontend, POOL_CONNECTION_POOL * vol
 	if (backend == NULL)
 		return false;
 
-	sp = MASTER_CONNECTION(backend)->sp;
+	sp = MAIN_CONNECTION(backend)->sp;
 
 	/*
 	 * cach connection if connection cache configuration parameter is enabled
@@ -515,10 +525,8 @@ backend_cleanup(POOL_CONNECTION * volatile *frontend, POOL_CONNECTION_POOL * vol
 	if (cache_connection)
 	{
 		/*
-		 * For those special databases, or when frontend client exits
-		 * abnormally, we don't cache connection to backend.  Also we have to
-		 * check whether reset query failed. If so, the existing connection to
-		 * backend may not be used and we don't want to cache the connection.
+		 * For those special databases, and when frontend client exits
+		 * abnormally, we don't cache connection to backend.
 		 */
 		if ((sp &&
 			 (!strcmp(sp->database, "template0") ||
@@ -527,8 +535,7 @@ backend_cleanup(POOL_CONNECTION * volatile *frontend, POOL_CONNECTION_POOL * vol
 			  !strcmp(sp->database, "regression"))) ||
 			(*frontend != NULL &&
 			 ((*frontend)->socket_state == POOL_SOCKET_EOF ||
-			  (*frontend)->socket_state == POOL_SOCKET_ERROR)) ||
-			reset_query_error)
+			  (*frontend)->socket_state == POOL_SOCKET_ERROR)))
 			cache_connection = false;
 	}
 
@@ -536,12 +543,8 @@ backend_cleanup(POOL_CONNECTION * volatile *frontend, POOL_CONNECTION_POOL * vol
 	 * Close frontend connection
 	 */
 	reset_connection();
-
-	if (*frontend != NULL)
-	{
-		pool_close(*frontend);
-		*frontend = NULL;
-	}
+	pool_close(*frontend);
+	*frontend = NULL;
 
 	if (cache_connection == false)
 	{
@@ -690,11 +693,7 @@ read_startup_packet(POOL_CONNECTION * cp)
 				}
 				else
 				{
-					ereport(DEBUG1,
-							(errmsg("reading startup packet"),
-							 errdetail("guc name: %s value: %s", p, p+strlen(p)+1)));
 					p += (strlen(p) + 1);
-
 				}
 
 				p += (strlen(p) + 1);
@@ -704,7 +703,7 @@ read_startup_packet(POOL_CONNECTION * cp)
 		case 1234:				/* cancel or SSL request */
 			/* set dummy database, user info */
 			sp->database = palloc0(1);
-			sp->user = palloc0(1);
+			sp->user = palloc(1);
 			break;
 
 		default:
@@ -743,18 +742,6 @@ read_startup_packet(POOL_CONNECTION * cp)
 	return sp;
 }
 
-/*
- * send startup packet
- */
-void
-send_startup_packet(POOL_CONNECTION_POOL_SLOT * cp)
-{
-	int			len;
-
-	len = htonl(cp->sp->len + sizeof(len));
-	pool_write(cp->con, &len, sizeof(len));
-	pool_write_and_flush(cp->con, cp->sp->startup_packet, cp->sp->len);
-}
 
 /*
  * Reuse existing connection
@@ -822,8 +809,8 @@ connect_using_existing_connection(POOL_CONNECTION * frontend,
 				if (VALID_BACKEND(i))
 					if (do_command(frontend, CONNECTION(backend, i),
 								   command_buf, MAJOR(backend),
-								   MASTER_CONNECTION(backend)->pid,
-								   MASTER_CONNECTION(backend)->key, 0) != POOL_CONTINUE)
+								   MAIN_CONNECTION(backend)->pid,
+								   MAIN_CONNECTION(backend)->key, 0) != POOL_CONTINUE)
 					{
 						ereport(ERROR,
 								(errmsg("unable to process command for backend connection"),
@@ -831,7 +818,8 @@ connect_using_existing_connection(POOL_CONNECTION * frontend,
 					}
 			}
 
-			pool_add_param(&MASTER(backend)->params, "application_name", sp->application_name);
+			pool_add_param(&MAIN(backend)->params, "application_name", sp->application_name);
+			set_application_name_with_string(sp->application_name);
 		}
 
 		send_params(frontend, backend);
@@ -847,7 +835,7 @@ connect_using_existing_connection(POOL_CONNECTION * frontend,
 
 		len = htonl(5);
 		pool_write(frontend, &len, sizeof(len));
-		tstate = TSTATE(backend, MASTER_NODE_ID);
+		tstate = TSTATE(backend, MAIN_NODE_ID);
 		pool_write(frontend, &tstate, 1);
 	}
 
@@ -1019,8 +1007,9 @@ static POOL_CONNECTION_POOL * connect_backend(StartupPacket *sp, POOL_CONNECTION
 
 		for (i = 0; i < NUM_BACKENDS; i++)
 		{
-			if (VALID_BACKEND(i) && CONNECTION_SLOT(backend, i))
+			if (VALID_BACKEND(i))
 			{
+
 				/* set DB node id */
 				pool_set_db_node_id(CONNECTION(backend, i), i);
 
@@ -1078,10 +1067,10 @@ static RETSIGTYPE die(int sig)
 {
 	int			save_errno;
 
-	save_errno = errno;
-
 	POOL_SETMASK(&BlockSig);
 
+	save_errno = errno;
+
 #ifdef NOT_USED
 	ereport(LOG,
 			(errmsg("child process received shutdown request signal %d", sig)));
@@ -1126,9 +1115,8 @@ static RETSIGTYPE die(int sig)
 			break;
 	}
 
-	POOL_SETMASK(&UnBlockSig);
-
 	errno = save_errno;
+	POOL_SETMASK(&UnBlockSig);
 }
 
 /*
@@ -1142,7 +1130,6 @@ static RETSIGTYPE close_idle_connection(int sig)
 	POOL_CONNECTION_POOL *p = pool_connection_pool;
 	ConnectionInfo *info;
 	int			save_errno = errno;
-	int			main_node_id;
 
 	/*
 	 * DROP DATABSE is ongoing.
@@ -1150,35 +1137,44 @@ static RETSIGTYPE close_idle_connection(int sig)
 	if (ignore_sigusr1)
 		return;
 
+#ifdef NOT_USED
+	ereport(DEBUG1,
+			(errmsg("close connection request received")));
+#endif
+
 	for (j = 0; j < pool_config->max_pool; j++, p++)
 	{
-		main_node_id = in_use_backend_id(p);
-		if (main_node_id < 0)
+		if (!MAIN_CONNECTION(p))
+			continue;
+		if (!MAIN_CONNECTION(p)->sp)
+			continue;
+		if (MAIN_CONNECTION(p)->sp->user == NULL)
 			continue;
 
-		if (!CONNECTION_SLOT(p, main_node_id))
-			continue;
-		if (!CONNECTION_SLOT(p, main_node_id)->sp)
-			continue;
-		if (CONNECTION_SLOT(p, main_node_id)->sp->user == NULL)
-			continue;
-
-		if (CONNECTION_SLOT(p, main_node_id)->closetime > 0)	/* idle connection? */
+		if (MAIN_CONNECTION(p)->closetime > 0)	/* idle connection? */
 		{
-			bool	freed = false;
+#ifdef NOT_USED
+			ereport(DEBUG1,
+					(errmsg("closing idle connection"),
+					 errdetail("user: %s database: %s", MAIN_CONNECTION(p)->sp->user, MAIN_CONNECTION(p)->sp->database)));
+#endif
 
 			pool_send_frontend_exits(p);
 
 			for (i = 0; i < NUM_BACKENDS; i++)
 			{
-				if (!CONNECTION_SLOT(p, i))
+				if (!VALID_BACKEND(i))
 					continue;
 
-				if (!freed)
+				if (i == 0)
 				{
+					/*
+					 * only first backend allocated the memory for the start
+					 * up packet
+					 */
 					pool_free_startup_packet(CONNECTION_SLOT(p, i)->sp);
 					CONNECTION_SLOT(p, i)->sp = NULL;
-					freed = true;
+
 				}
 				pool_close(CONNECTION(p, i));
 			}
@@ -1224,34 +1220,6 @@ disable_authentication_timeout(void)
 	}
 }
 
-/*
- * send frontend exiting messages to all connections.  this is called
- * in any case when child process exits, for example failover, child
- * life time expires or child max connections expires.
- */
-static void
-send_frontend_exits(void)
-{
-	int			i;
-	POOL_CONNECTION_POOL *p = pool_connection_pool;
-
-	pool_sigset_t oldmask;
-
-	POOL_SETMASK2(&BlockSig, &oldmask);
-
-	for (i = 0; i < pool_config->max_pool; i++, p++)
-	{
-		if (!MASTER_CONNECTION(p))
-			continue;
-		if (!MASTER_CONNECTION(p)->sp)
-			continue;
-		if (MASTER_CONNECTION(p)->sp->user == NULL)
-			continue;
-		pool_send_frontend_exits(p);
-	}
-
-	POOL_SETMASK(&oldmask);
-}
 
 static void
 send_params(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend)
@@ -1263,7 +1231,7 @@ send_params(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend)
 				sendlen;
 
 	index = 0;
-	while (pool_get_param(&MASTER(backend)->params, index++, &name, &value) == 0)
+	while (pool_get_param(&MAIN(backend)->params, index++, &name, &value) == 0)
 	{
 		pool_write(frontend, "S", 1);
 		len = sizeof(sendlen) + strlen(name) + 1 + strlen(value) + 1;
@@ -1281,22 +1249,6 @@ send_params(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend)
 	}
 }
 
-void
-pool_free_startup_packet(StartupPacket *sp)
-{
-	if (sp)
-	{
-		if (sp->startup_packet)
-			pfree(sp->startup_packet);
-		if (sp->database)
-			pfree(sp->database);
-		if (sp->user)
-			pfree(sp->user);
-		pfree(sp);
-	}
-	sp = NULL;
-}
-
 /*
  * Do house keeping works when pgpool child process exits
  */
@@ -1313,7 +1265,16 @@ child_will_go_down(int code, Datum arg)
 
 	/* count down global connection counter */
 	if (accepted)
+	{
 		connection_count_down();
+		if (pool_config->log_disconnections)
+		{
+			if (child_frontend)
+				log_disconnections(child_frontend->database, child_frontend->username);
+			else
+				log_disconnections("","");
+		}
+	}
 
 	if ((pool_config->memory_cache_enabled || pool_config->enable_shared_relcache)
 		&& !pool_is_shmem_cache())
@@ -1323,7 +1284,7 @@ child_will_go_down(int code, Datum arg)
 
 	/* let backend know now we are exiting */
 	if (pool_connection_pool)
-		send_frontend_exits();
+		close_all_backend_connections();
 }
 void
 child_exit(int code)
@@ -1345,215 +1306,7 @@ child_exit(int code)
 	exit(code);
 }
 
-/*
- * create a persistent connection
- */
-POOL_CONNECTION_POOL_SLOT *
-make_persistent_db_connection(
-							  int db_node_id, char *hostname, int port, char *dbname, char *user, char *password, bool retry)
-{
-	POOL_CONNECTION_POOL_SLOT *cp;
-	int			fd;
 
-#define MAX_USER_AND_DATABASE	1024
-
-	/* V3 startup packet */
-	typedef struct
-	{
-		int			protoVersion;
-		char		data[MAX_USER_AND_DATABASE];
-	}			StartupPacket_v3;
-
-	static StartupPacket_v3 * startup_packet;
-	int			len,
-				len1;
-
-	cp = palloc0(sizeof(POOL_CONNECTION_POOL_SLOT));
-	startup_packet = palloc0(sizeof(*startup_packet));
-	startup_packet->protoVersion = htonl(0x00030000);	/* set V3 proto
-														 * major/minor */
-
-	/*
-	 * create socket
-	 */
-	if (*hostname == '/')
-	{
-		fd = connect_unix_domain_socket_by_port(port, hostname, retry);
-	}
-	else
-	{
-		fd = connect_inet_domain_socket_by_port(hostname, port, retry);
-	}
-
-	if (fd < 0)
-	{
-		free_persisten_db_connection_memory(cp);
-		pfree(startup_packet);
-		ereport(ERROR,
-				(errmsg("failed to make persistent db connection"),
-				 errdetail("connection to host:\"%s:%d\" failed", hostname, port)));
-	}
-
-	cp->con = pool_open(fd, true);
-	cp->closetime = 0;
-	cp->con->isbackend = 1;
-	pool_set_db_node_id(cp->con, db_node_id);
-
-	pool_ssl_negotiate_clientserver(cp->con);
-
-	/*
-	 * build V3 startup packet
-	 */
-	len = snprintf(startup_packet->data, sizeof(startup_packet->data), "user") + 1;
-	len1 = snprintf(&startup_packet->data[len], sizeof(startup_packet->data) - len, "%s", user) + 1;
-	if (len1 >= (sizeof(startup_packet->data) - len))
-	{
-		pool_close(cp->con);
-		free_persisten_db_connection_memory(cp);
-		pfree(startup_packet);
-		ereport(ERROR,
-				(errmsg("failed to make persistent db connection"),
-				 errdetail("user name is too long")));
-	}
-
-	len += len1;
-	len1 = snprintf(&startup_packet->data[len], sizeof(startup_packet->data) - len, "database") + 1;
-	if (len1 >= (sizeof(startup_packet->data) - len))
-	{
-		pool_close(cp->con);
-		free_persisten_db_connection_memory(cp);
-		pfree(startup_packet);
-		ereport(ERROR,
-				(errmsg("failed to make persistent db connection"),
-				 errdetail("user name is too long")));
-	}
-
-	len += len1;
-	len1 = snprintf(&startup_packet->data[len], sizeof(startup_packet->data) - len, "%s", dbname) + 1;
-	if (len1 >= (sizeof(startup_packet->data) - len))
-	{
-		pool_close(cp->con);
-		free_persisten_db_connection_memory(cp);
-		pfree(startup_packet);
-		ereport(ERROR,
-				(errmsg("failed to make persistent db connection"),
-				 errdetail("database name is too long")));
-	}
-	len += len1;
-	startup_packet->data[len++] = '\0';
-
-	cp->sp = palloc(sizeof(StartupPacket));
-
-	cp->sp->startup_packet = (char *) startup_packet;
-	cp->sp->len = len + 4;
-	cp->sp->major = 3;
-	cp->sp->minor = 0;
-	cp->sp->database = pstrdup(dbname);
-	cp->sp->user = pstrdup(user);
-
-	/*
-	 * send startup packet
-	 */
-	PG_TRY();
-	{
-		send_startup_packet(cp);
-		connection_do_auth(cp, password);
-	}
-	PG_CATCH();
-	{
-		pool_close(cp->con);
-		free_persisten_db_connection_memory(cp);
-		PG_RE_THROW();
-	}
-	PG_END_TRY();
-
-	return cp;
-}
-
-/*
- * make_persistent_db_connection_noerror() is a wrapper over
- * make_persistent_db_connection() which does not ereports in case of an error
- */
-POOL_CONNECTION_POOL_SLOT *
-make_persistent_db_connection_noerror(
-									  int db_node_id, char *hostname, int port, char *dbname, char *user, char *password, bool retry)
-{
-	POOL_CONNECTION_POOL_SLOT *slot = NULL;
-	MemoryContext oldContext = CurrentMemoryContext;
-
-	PG_TRY();
-	{
-		slot = make_persistent_db_connection(db_node_id,
-											 hostname,
-											 port,
-											 dbname,
-											 user,
-											 password, retry);
-	}
-	PG_CATCH();
-	{
-		EmitErrorReport();
-		MemoryContextSwitchTo(oldContext);
-		FlushErrorState();
-		slot = NULL;
-	}
-	PG_END_TRY();
-	return slot;
-}
-
-/*
- * Free memory of POOL_CONNECTION_POOL_SLOT.  Should only be used in
- * make_persistent_db_connection and discard_persistent_db_connection.
- */
-void
-free_persisten_db_connection_memory(POOL_CONNECTION_POOL_SLOT * cp)
-{
-	if (!cp)
-		return;
-	if (!cp->sp)
-	{
-		pfree(cp);
-		return;
-	}
-	if (cp->sp->startup_packet)
-		pfree(cp->sp->startup_packet);
-	if (cp->sp->database)
-		pfree(cp->sp->database);
-	if (cp->sp->user)
-		pfree(cp->sp->user);
-	pfree(cp->sp);
-	pfree(cp);
-}
-
-/*
- * Discard connection and memory allocated by
- * make_persistent_db_connection().
- */
-void
-discard_persistent_db_connection(POOL_CONNECTION_POOL_SLOT * cp)
-{
-	int			len;
-
-	if (cp == NULL)
-		return;
-
-	pool_write(cp->con, "X", 1);
-	len = htonl(4);
-	pool_write(cp->con, &len, sizeof(len));
-
-	/*
-	 * XXX we cannot call pool_flush() here since backend may already close
-	 * the socket and pool_flush() automatically invokes fail over handler.
-	 * This could happen in copy command (remember the famous "lost
-	 * synchronization with server, resetting connection" message)
-	 */
-	pool_set_nonblock(cp->con->fd);
-	pool_flush_it(cp->con);
-	pool_unset_nonblock(cp->con->fd);
-
-	pool_close(cp->con);
-	free_persisten_db_connection_memory(cp);
-}
 
 /*
  * Count up connection counter (from frontend to pgpool) in shared memory and
@@ -1606,188 +1359,11 @@ connection_count_down(void)
  */
 static RETSIGTYPE wakeup_handler(int sig)
 {
+	sigusr2_received = 1;
 }
 
 
-/*
- * Select load balancing node. This function is called when:
- * 1) client connects
- * 2) the node previously selected for the load balance node is down
- */
-int
-select_load_balancing_node(void)
-{
-	int			selected_slot;
-	double		total_weight,
-				r;
-	int			i;
-	int			index_db = -1,
-				index_app = -1;
-	POOL_SESSION_CONTEXT *ses = pool_get_session_context(false);
-	int			tmp;
-	int			no_load_balance_node_id = -2;
 
-	/*
-	 * -2 indicates there's no database_redirect_preference_list. -1 indicates
-	 * database_redirect_preference_list exists and any of standby nodes
-	 * specified.
-	 */
-	int			suggested_node_id = -2;
-
-#if defined(sun) || defined(__sun)
-	r = (((double) rand()) / RAND_MAX);
-#else
-	r = (((double) random()) / RAND_MAX);
-#endif
-
-	/*
-	 * Check database_redirect_preference_list
-	 */
-	if (SL_MODE && pool_config->redirect_dbnames)
-	{
-		char	   *database = MASTER_CONNECTION(ses->backend)->sp->database;
-
-		/*
-		 * Check to see if the database matches any of
-		 * database_redirect_preference_list
-		 */
-		index_db = regex_array_match(pool_config->redirect_dbnames, database);
-		if (index_db >= 0)
-		{
-			/* Matches */
-			ereport(DEBUG1,
-					(errmsg("selecting load balance node db matched"),
-					 errdetail("dbname: %s index is %d dbnode is %s weight is %f", database, index_db,
-							   pool_config->db_redirect_tokens->token[index_db].right_token,
-							   pool_config->db_redirect_tokens->token[index_db].weight_token)));
-
-			tmp = choose_db_node_id(pool_config->db_redirect_tokens->token[index_db].right_token);
-			if (tmp == -1 || (tmp >= 0 && VALID_BACKEND_RAW(tmp)))
-				suggested_node_id = tmp;
-		}
-	}
-
-	/*
-	 * Check app_name_redirect_preference_list
-	 */
-	if (SL_MODE && pool_config->redirect_app_names)
-	{
-		char	   *app_name = MASTER_CONNECTION(ses->backend)->sp->application_name;
-
-		/*
-		 * Check only if application name is set. Old applications may not
-		 * have application name.
-		 */
-		if (app_name && strlen(app_name) > 0)
-		{
-			/*
-			 * Check to see if the aplication name matches any of
-			 * app_name_redirect_preference_list.
-			 */
-			index_app = regex_array_match(pool_config->redirect_app_names, app_name);
-			if (index_app >= 0)
-			{
-
-				/*
-				 * if the aplication name matches any of
-				 * app_name_redirect_preference_list,
-				 * database_redirect_preference_list will be ignored.
-				 */
-				index_db = -1;
-
-				/* Matches */
-				ereport(DEBUG1,
-						(errmsg("selecting load balance node db matched"),
-						 errdetail("app_name: %s index is %d dbnode is %s weight is %f", app_name, index_app,
-								   pool_config->app_name_redirect_tokens->token[index_app].right_token,
-								   pool_config->app_name_redirect_tokens->token[index_app].weight_token)));
-
-				tmp = choose_db_node_id(pool_config->app_name_redirect_tokens->token[index_app].right_token);
-				if (tmp == -1 || (tmp >= 0 && VALID_BACKEND_RAW(tmp)))
-					suggested_node_id = tmp;
-			}
-		}
-	}
-
-	if (suggested_node_id >= 0)
-	{
-		/*
-		 * If the weight is bigger than random rate then send to
-		 * suggested_node_id. If the weight is less than random rate then
-		 * choose load balance node from other nodes.
-		 */
-		if ((index_db >= 0 && r <= pool_config->db_redirect_tokens->token[index_db].weight_token) ||
-			(index_app >= 0 && r <= pool_config->app_name_redirect_tokens->token[index_app].weight_token))
-		{
-			ereport(DEBUG1,
-					(errmsg("selecting load balance node"),
-					 errdetail("selected backend id is %d", suggested_node_id)));
-			return suggested_node_id;
-		}
-		else
-			no_load_balance_node_id = suggested_node_id;
-	}
-
-	/* In case of sending to standby */
-	if (suggested_node_id == -1)
-	{
-		/* If the weight is less than random rate then send to primary. */
-		if ((index_db >= 0 && r > pool_config->db_redirect_tokens->token[index_db].weight_token) ||
-			(index_app >= 0 && r > pool_config->app_name_redirect_tokens->token[index_app].weight_token))
-		{
-			ereport(DEBUG1,
-					(errmsg("selecting load balance node"),
-					 errdetail("selected backend id is %d", PRIMARY_NODE_ID)));
-			return PRIMARY_NODE_ID;
-		}
-	}
-
-	/* Choose a backend in random manner with weight */
-	selected_slot = MASTER_NODE_ID;
-	total_weight = 0.0;
-
-	for (i = 0; i < NUM_BACKENDS; i++)
-	{
-		if (VALID_BACKEND_RAW(i))
-		{
-			if (i == no_load_balance_node_id)
-				continue;
-			if (suggested_node_id == -1)
-			{
-				if (i != PRIMARY_NODE_ID)
-					total_weight += BACKEND_INFO(i).backend_weight;
-			}
-			else
-				total_weight += BACKEND_INFO(i).backend_weight;
-		}
-	}
-
-#if defined(sun) || defined(__sun)
-	r = (((double) rand()) / RAND_MAX) * total_weight;
-#else
-	r = (((double) random()) / RAND_MAX) * total_weight;
-#endif
-
-	total_weight = 0.0;
-	for (i = 0; i < NUM_BACKENDS; i++)
-	{
-		if ((suggested_node_id == -1 && i == PRIMARY_NODE_ID) || i == no_load_balance_node_id)
-			continue;
-
-		if (VALID_BACKEND_RAW(i) && BACKEND_INFO(i).backend_weight > 0.0)
-		{
-			if (r >= total_weight)
-				selected_slot = i;
-			else
-				break;
-			total_weight += BACKEND_INFO(i).backend_weight;
-		}
-	}
-	ereport(DEBUG1,
-			(errmsg("selecting load balance node"),
-			 errdetail("selected backend id is %d", selected_slot)));
-	return selected_slot;
-}
 
 /* SIGHUP handler */
 static RETSIGTYPE reload_config_handler(int sig)
@@ -1816,7 +1392,7 @@ check_stop_request(void)
 }
 
 /*
- * Initialize my backend status and master node id.
+ * Initialize my backend status and main node id.
  * We copy the backend status to private area so that
  * they are not changed while I am alive.
  */
@@ -1835,7 +1411,7 @@ pool_initialize_private_backend_status(void)
 		my_backend_status[i] = &private_backend_status[i];
 	}
 
-	my_master_node_id = REAL_MASTER_NODE_ID;
+	my_main_node_id = REAL_MAIN_NODE_ID;
 }
 
 static void
@@ -1888,7 +1464,7 @@ wait_for_new_connections(int *fds, struct timeval *timeout, SockAddr *saddr)
 				tmback = {0, 0};
 
 	for (walk = fds; *walk != -1; walk++)
-		pool_set_nonblock(*walk);
+		socket_set_nonblock(*walk);
 
 	if (SERIALIZE_ACCEPT)
 		set_ps_display("wait for accept lock", false);
@@ -2114,7 +1690,7 @@ retry_accept:
 	/*
 	 * Make sure that the socket is non blocking.
 	 */
-	pool_unset_nonblock(afd);
+	socket_unset_nonblock(afd);
 
 #ifdef ACCEPT_PERFORMANCE
 	gettimeofday(&now2, 0);
@@ -2141,10 +1717,9 @@ check_config_reload(void)
 		if (pool_config->enable_pool_hba)
 		{
 			load_hba(get_hba_file_name());
+			if (strcmp("", pool_config->pool_passwd))
+				pool_reopen_passwd_file();
 		}
-		if (strcmp("", pool_config->pool_passwd))
-			pool_reopen_passwd_file();
-
 		got_sighup = 0;
 	}
 }
@@ -2408,7 +1983,7 @@ retry_startup:
 		 * however we should make sure that the startup packet contents are
 		 * identical. OPTION data and others might be different.
 		 */
-		if (sp->len != MASTER_CONNECTION(backend)->sp->len)
+		if (sp->len != MAIN_CONNECTION(backend)->sp->len)
 		{
 			ereport(DEBUG1,
 					(errmsg("selecting backend connection"),
@@ -2416,7 +1991,7 @@ retry_startup:
 
 			found = 0;
 		}
-		else if (memcmp(sp->startup_packet, MASTER_CONNECTION(backend)->sp->startup_packet, sp->len) != 0)
+		else if (memcmp(sp->startup_packet, MAIN_CONNECTION(backend)->sp->startup_packet, sp->len) != 0)
 		{
 			ereport(DEBUG1,
 					(errmsg("selecting backend connection"),
@@ -2451,6 +2026,34 @@ retry_startup:
 	return backend;
 }
 
+static void
+log_disconnections(char *database, char *username)
+{
+	struct timeval endTime;
+	long diff;
+	long		secs;
+	int			msecs,
+				hours,
+				minutes,
+				seconds;
+
+	gettimeofday(&endTime, NULL);
+	diff = (long) ((endTime.tv_sec - startTime.tv_sec) * 1000000  + (endTime.tv_usec - startTime.tv_usec));
+
+	msecs = (int) (diff % 1000000) / 1000;
+	secs = (long) (diff / 1000000);
+	hours = secs / 3600;
+	secs %= 3600;
+	minutes = secs / 60;
+	seconds = secs % 60;
+
+	ereport(LOG,
+			(errmsg("frontend disconnection: session time: %d:%02d:%02d.%03d "
+					"user=%s database=%s host=%s%s%s",
+					hours, minutes, seconds, msecs,
+					username, database, remote_host, remote_port[0] ? " port=" : "", remote_port)));
+}
+
 static void
 print_process_status(char *remote_host, char *remote_port)
 {
@@ -2460,49 +2063,7 @@ print_process_status(char *remote_host, char *remote_port)
 		snprintf(remote_ps_data, sizeof(remote_ps_data), "%s(%s)", remote_host, remote_port);
 }
 
-bool
-is_session_connected()
-{
-	if (processType == PT_CHILD)
-		return (pool_get_session_context(true) != NULL);
-	return false;
-}
 
-/*
- * Given db node specified in pgpool.conf, returns appropriate physical
- * DB node id.
- * Acceptable db node specifications are:
- *
- * primary: primary node
- * standby: any of standby node
- * numeric: physical node id
- *
- * If specified node does exist, returns MASTER_NODE_ID.  If "standby" is
- * specified, returns -1. Caller should choose one of standby nodes
- * appropriately.
- */
-static int
-choose_db_node_id(char *str)
-{
-	int			node_id = MASTER_NODE_ID;
-
-	if (!strcmp("primary", str) && PRIMARY_NODE_ID >= 0)
-	{
-		node_id = PRIMARY_NODE_ID;
-	}
-	else if (!strcmp("standby", str))
-	{
-		node_id = -1;
-	}
-	else
-	{
-		int			tmp = atoi(str);
-
-		if (tmp >= 0 && tmp < NUM_BACKENDS)
-			node_id = tmp;
-	}
-	return node_id;
-}
 
 int
 send_to_pg_frontend(char *data, int len, bool flush)
@@ -2527,9 +2088,9 @@ set_pg_frontend_blocking(bool blocking)
 	if (child_frontend->socket_state != POOL_SOCKET_VALID)
 		return -1;
 	if (blocking)
-		pool_unset_nonblock(child_frontend->fd);
+		socket_unset_nonblock(child_frontend->fd);
 	else
-		pool_set_nonblock(child_frontend->fd);
+		socket_set_nonblock(child_frontend->fd);
 	return 0;
 }
 
@@ -2554,155 +2115,4 @@ static int opt_sort(const void *a, const void *b)
 	return strcmp( *(char **)a, *(char **)b);
 }
 
-/*
- * Returns PostgreSQL version.
- * The returned PgVersion struct is in static memory.
- * Caller must not modify it.
- *
- * Note:
- * Must be called while query context already exists.
- * If there's something goes wrong, this raises FATAL. So never returns to caller.
- *
- */
-PGVersion *
-Pgversion(POOL_CONNECTION_POOL * backend)
-{
-#define VERSION_BUF_SIZE	10
-	static	PGVersion	pgversion;
-	static	POOL_RELCACHE *relcache;
-	char	*result;
-	char	*p;
-	char	buf[VERSION_BUF_SIZE];
-	int		i;
-	int		major;
-	int		minor;
 
-	/*
-	 * First, check local cache. If cache is set, just return it.
-	 */
-	if (pgversion.major != 0)
-	{
-		ereport(DEBUG5,
-				(errmsg("Pgversion: local cache returned")));
-
-		return &pgversion;
-	}
-
-	if (!relcache)
-	{
-		/*
-		 * Create relcache.
-		 */
-		relcache = pool_create_relcache(pool_config->relcache_size, "SELECT version()",
-										string_register_func, string_unregister_func, false);
-		if (relcache == NULL)
-		{
-			ereport(FATAL,
-					(errmsg("Pgversion: unable to create relcache while getting PostgreSQL version.")));
-			return NULL;
-		}
-	}
-
-	/*
-	 * Search relcache.
-	 */
-	result = (char *)pool_search_relcache(relcache, backend, "version");
-	if (result == 0)
-	{
-		ereport(FATAL,
-				(errmsg("Pgversion: unable to search relcache while getting PostgreSQL version.")));
-		return NULL;
-	}
-
-	ereport(DEBUG5,
-			(errmsg("Pgversion: version string: %s", result)));
-
-	/*
-	 * Extract major version number.  We create major version as "version" *
-	 * 10.  For example, for V10, the major version number will be 100, for
-	 * V9.6 it will be 96, and so on.  For alpha or beta version, the version
-	 * string could be something like "12beta1". In this case we assume that
-	 * atoi(3) is smart enough to stop at the first character which is not a
-	 * valid digit (in our case 'b')). So "12beta1" should be converted to 12.
-	 */
-	p = strchr(result, ' ');
-	if (p == NULL)
-	{
-		ereport(FATAL,
-				(errmsg("Pgversion: unable to find the first space in the version string: %s", result)));
-		return NULL;
-	}
-
-	p++;
-	i = 0;
-	while (i < VERSION_BUF_SIZE - 1 && p && *p != '.')
-	{
-		buf[i++] = *p++;
-	}
-	buf[i] = '\0';
-	major = atoi(buf);
-	ereport(DEBUG5,
-			(errmsg("Pgversion: major version: %d", major)));
-
-	/* Assuming PostgreSQL V100 is the final release:-) */
-	if (major < 6 || major > 100)
-	{
-		ereport(FATAL,
-				(errmsg("Pgversion: wrong major version: %d", major)));
-		return NULL;
-	}
-
-	/*
-	 * If major version is 10 or above, we are done to extract major.
-	 * Otherwise extract below decimal point part.
-	 */
-	if (major >= 10)
-	{
-		major *= 10;
-	}
-	else
-	{
-		p++;
-		i = 0;
-		while (i < VERSION_BUF_SIZE -1 && p && *p != '.' && *p != ' ')
-		{
-			buf[i++] = *p++;
-		}
-		buf[i] = '\0';
-		major = major * 10 + atoi(buf);
-		ereport(DEBUG5,
-				(errmsg("Pgversion: major version: %d", major)));
-		pgversion.major = major;
-	}
-
-	/*
-	 * Extract minor version.
-	 */
-	p++;
-	i = 0;
-	while (i < VERSION_BUF_SIZE -1 && p && *p != '.' && *p != ' ')
-	{
-		buf[i++] = *p++;
-	}
-	buf[i] = '\0';
-	minor = atoi(buf);
-	ereport(DEBUG5,
-			(errmsg("Pgversion: minor version: %d", minor)));
-
-	if (minor < 0 || minor > 100)
-	{
-		ereport(FATAL,
-				(errmsg("Pgversion: wrong minor version: %d", minor)));
-		return NULL;
-	}
-
-
-	/*
-	 * Ok, everything looks good. Set the local cache.
-	 */
-	pgversion.major = major;
-	pgversion.minor = minor;
-	strncpy(pgversion.version_string, result, sizeof(pgversion.version_string) - 1);
-
-	return &pgversion;
-}
