@@ -6,7 +6,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2015	PgPool Global Development Group
+ * Copyright (c) 2003-2020	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -44,15 +44,18 @@
 #endif
 
 #include "pool.h"
+#include "pool_config.h"
+#include "utils/pool_signal.h"
 #include "utils/palloc.h"
 #include "utils/memutils.h"
 #include "utils/elog.h"
-#include "pool_config.h"
+#include "utils/ps_status.h"
 #include "auth/md5.h"
 #include "watchdog/watchdog.h"
 #include "watchdog/wd_lifecheck.h"
 #include "watchdog/wd_utils.h"
 
+
 #define MAX_BIND_TRIES 5
 /*
  * heartbeat packet
@@ -360,7 +363,7 @@ wd_hb_receiver(int fork_wait_time, WdHbIf * hb_if)
 	}
 
 	on_exit_reset();
-	processType = PT_HB_RECEIVER;
+	SetProcessGlobalVaraibles(PT_HB_RECEIVER);
 
 	if (fork_wait_time > 0)
 	{
@@ -493,7 +496,7 @@ wd_hb_sender(int fork_wait_time, WdHbIf * hb_if)
 	}
 
 	on_exit_reset();
-	processType = PT_HB_SENDER;
+	SetProcessGlobalVaraibles(PT_HB_SENDER);
 
 	if (fork_wait_time > 0)
 	{
@@ -547,7 +550,7 @@ wd_hb_sender(int fork_wait_time, WdHbIf * hb_if)
 
 		/* contents of packet */
 		gettimeofday(&pkt.send_time, NULL);
-		strlcpy(pkt.from, pool_config->wd_hostname, sizeof(pkt.from));
+		strlcpy(pkt.from, pool_config->wd_nodes.wd_node_info[pool_config->pgpool_node_id].hostname, sizeof(pkt.from));
 		pkt.from_pgpool_port = pool_config->port;
 
 		/* authentication key */
